// Fill out your copyright notice in the Description page of Project Settings.


#include "Player/LMPlayer.h"

#include "KismetTraceUtils.h"
#include "LMBulletInfo.h"
#include "Camera/CameraComponent.h"
#include "Weapon/LMAmmo.h"
#include "Weapon/LMWeaponManager.h"

ALMPlayer::ALMPlayer()
{
 	// Set this character to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	Camera = CreateDefaultSubobject<UCameraComponent>(TEXT("Camera"));
	Camera->SetupAttachment(RootComponent);
	Camera->bUsePawnControlRotation = true;
	Camera->SetRelativeLocation(FVector(20, 0, 90));
	
	ArmsMesh = CreateDefaultSubobject<USkeletalMeshComponent>(TEXT("Arms Mesh"));
	ArmsMesh->SetupAttachment(Camera);
}

// Called when the game starts or when spawned
void ALMPlayer::BeginPlay()
{
	Super::BeginPlay();
	
}

void ALMPlayer::PickUpAmmo(ALMAmmo* Ammo)
{
	if (GEngine)
	{
		FString AmmoAmount = FString::Printf(TEXT("Quantité de munition : %i"), Ammo->GetAmmoAmount());
		GEngine->AddOnScreenDebugMessage(1, 30.f, FColor::Red, AmmoAmount);
	}
	WeaponManager->AddAmmoType(Ammo->GetAmmoType(), Ammo->GetAmmoAmount());
}

void ALMPlayer::SetWeaponManager(ULMWeaponManager* Manager)
{
	UE_LOG(LogTemp, Warning, TEXT("Set weapon manager"))
	if (!Manager)
	{
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 10.f, FColor::Red, TEXT("No Weapon Manager was given to the player !"));
		return;
	}
	
	WeaponManager = Manager;
	WeaponManager->Initialize(ArmsMesh);
}

void ALMPlayer::PlayAnimation(UAnimMontage* Animation)
{
}

void ALMPlayer::FireBullets(const FLMBulletInfo Settings)
{
	FVector EndLocation = Settings.Origin + (Settings.Direction * Settings.MaxDistance);
	TArray<FHitResult> Hits;
	DrawDebugLineTraceMulti(GetWorld(), Settings.Origin, EndLocation, EDrawDebugTrace::ForDuration, true, Hits, FColor::Green, FColor::Red, 10.f);
}

void ALMPlayer::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

void ALMPlayer::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
	Super::SetupPlayerInputComponent(PlayerInputComponent);

}

