﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "Weapon/LMWeaponManager.h"

#include "Weapon/LMWeaponBase.h"


ULMWeaponManager::ULMWeaponManager()
{
	PrimaryComponentTick.bCanEverTick = false;
}

void ULMWeaponManager::BeginPlay()
{
	Super::BeginPlay();
}

void ULMWeaponManager::Initialize(USkeletalMeshComponent* Mesh)
{
	ArmsMesh = Mesh;
	
	for (auto WeaponTemplate : StartingWeapons)
	{
		if (!WeaponTemplate)
			continue;
		
		ALMWeaponBase* Instance = GetWorld()->SpawnActor<ALMWeaponBase>(WeaponTemplate);
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 2.f, FColor::Blue, TEXT("Spawing"));
		Instance->SetActorHiddenInGame(true);
		
		Weapons.Add(Instance);
		
		FAttachmentTransformRules Rules = FAttachmentTransformRules(EAttachmentRule::SnapToTarget,
		                                                            EAttachmentRule::KeepRelative,
		                                                            EAttachmentRule::KeepRelative,
		                                                            false);
		
		Instance->AttachToComponent(ArmsMesh, Rules);
	}
	
	if (!Weapons.IsEmpty())
	{
		SetWeapon(CurrentWeaponIndex);
	}
}

void ULMWeaponManager::AddAmmoType(EAmmoType AmmoType, int AmmoCount)
{
	FString Debug = FString::Printf(TEXT("Adding %i ammo of type %i"), AmmoCount, AmmoType);
	GEngine->AddOnScreenDebugMessage(1, 1.f, FColor::Cyan, Debug);

	
}

void ULMWeaponManager::Fire()
{
	ALMWeaponBase* Weapon = GetCurrentWeapon();
	
	GEngine->AddOnScreenDebugMessage(2, 1.f, FColor::Cyan, "Fire");
	Weapon->PrimaryFire();
}

void ULMWeaponManager::Reload()
{
	GEngine->AddOnScreenDebugMessage(3, 1.f, FColor::Cyan, "Reloading");
	
	ALMWeaponBase* Weapon = GetCurrentWeapon();
	Weapon->Reload();
}

void ULMWeaponManager::SwitchWeapon(const int Direction)
{
	const int NewIndex = Direction + CurrentWeaponIndex;
	if (NewIndex >= 0 && NewIndex < Weapons.Num())
	{
		SetWeapon(NewIndex);
	}
	else
	{
		if (NewIndex < 0)
			SetWeapon(Weapons.Num() - 1);
		else
			SetWeapon(0);
	}
}

void ULMWeaponManager::SetWeapon(const int Index)
{
	if (Index < 0 || Index >= Weapons.Num())
	{
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 5, FColor::Red, "Invalid Weapon Index");
		return;
	}

	GetCurrentWeapon()->SetActorHiddenInGame(true);
	CurrentWeaponIndex = Index;
	GetCurrentWeapon()->SetActorHiddenInGame(false);
}

